@@ -1,8 +1,8 @@
+import re
 import os
 import stat
 import sys
 import time
-import re
 from tempfile import mkdtemp, gettempdir
 from glob import glob
 from base64 import decodestring
@@ -46,10 +46,18 @@ _GIF_CONTENT_DIFFERENT = 'R0lGODlhBAABAJEAANHV3ufr7qy9xGyiyCH5BAAAAAAALAAAAAAEAA
 
 #TEST_MEDIA_ROOT = os.path.join(gettempdir(), 'fake_media_root')
 #_original_MEDIA_ROOT = settings.MEDIA_ROOT
-_original_DEBUG = settings.DEBUG
-_original_DJANGO_STATIC_SAVE_PREFIX = getattr(settings, 'DJANGO_STATIC_SAVE_PREFIX', '')
-_original_DJANGO_STATIC_NAME_PREFIX = getattr(settings, 'DJANGO_STATIC_NAME_PREFIX', '')
-_original_DJANGO_STATIC_MEDIA_URL = getattr(settings, 'DJANGO_STATIC_MEDIA_URL', '')
+_MISSING = id(get_library_wrapper) # get semi-random mark
+_saved_settings = []
+for name in [ "DEBUG",
+              "DJANGO_STATIC",
+              "DJANGO_STATIC_SAVE_PREFIX",
+              "DJANGO_STATIC_NAME_PREFIX",
+              "DJANGO_STATIC_MEDIA_URL",
+              "DJANGO_STATIC_FILE_PROXY",
+              "DJANGO_STATIC_USE_SYMLINK",
+              "DJANGO_STATIC_CLOSURE_COMPILER",
+              "DJANGO_STATIC_MEDIA_ROOTS" ]:
+    _saved_settings.append((name, getattr(settings, name, _MISSING)))
 
 class TestDjangoStatic(TestCase):
     
@@ -68,13 +76,26 @@ class TestDjangoStatic(TestCase):
         #    os.mkdir(TEST_MEDIA_ROOT)
             
         # All tests is going to run off this temp directory
-        settings.MEDIA_ROOT = mkdtemp()
-        self.__added_dirs.append(settings.MEDIA_ROOT)
+        settings.MEDIA_ROOT = self._mkdir()
         
-        # Disable Closure Compiler if set
+        # Set all django-static settings to known values so it isn't
+        # dependant on the real values in settings.py
+        settings.DJANGO_STATIC = True
+        settings.DJANGO_STATIC_SAVE_PREFIX = ""
+        settings.DJANGO_STATIC_NAME_PREFIX = ""
+        settings.DJANGO_STATIC_MEDIA_URL = ""
+        settings.DJANGO_STATIC_USE_SYMLINK = True
+        settings.DJANGO_STATIC_FILE_PROXY = None
         settings.DJANGO_STATIC_CLOSURE_COMPILER = None
+        if hasattr(settings, "DJANGO_STATIC_MEDIA_ROOTS"):
+            del settings.DJANGO_STATIC_MEDIA_ROOTS
         
         super(TestDjangoStatic, self).setUp()
+
+    def _mkdir(self):
+        dir = mkdtemp()
+        self.__added_dirs.append(dir)
+        return dir
         
     def tearDown(self):
         for filepath in self.__added_filepaths:
@@ -82,13 +103,11 @@ class TestDjangoStatic(TestCase):
                 os.remove(filepath)
                 
         # restore things for other potential tests
-        settings.DEBUG = _original_DEBUG
-        settings.DJANGO_STATIC_SAVE_PREFIX = _original_DJANGO_STATIC_SAVE_PREFIX
-        settings.DJANGO_STATIC_NAME_PREFIX = _original_DJANGO_STATIC_NAME_PREFIX
-        settings.DJANGO_STATIC_MEDIA_URL = _original_DJANGO_STATIC_MEDIA_URL
-        settings.DJANGO_STATIC_FILE_PROXY = None
-        if hasattr(settings, "DJANGO_STATIC_MEDIA_ROOTS"):
-            del settings.DJANGO_STATIC_MEDIA_ROOTS
+        for name, value in _saved_settings:
+            if value == _MISSING and hasattr(settings, name):
+                delattr(settings, name)
+            else:
+                setattr(settings, name, value)
         
         for dir in self.__added_dirs:
             if dir.startswith(gettempdir()):
@@ -274,11 +293,9 @@ class TestDjangoStatic(TestCase):
         """
         
         dir1 = settings.MEDIA_ROOT
-        dir2 = mkdtemp()
-        self.__added_dirs.append(dir2)
+        dir2 = self._mkdir()
         settings.DJANGO_STATIC_MEDIA_ROOTS = [ dir1, dir2 ]
-        dir3 = mkdtemp()
-        self.__added_dirs.append(dir3)
+        dir3 = self._mkdir()
 
         open(dir1 + '/img100.gif', 'w').write(_GIF_CONTENT)
         open(dir2 + '/img200.gif', 'w').write(_GIF_CONTENT)
@@ -318,11 +335,9 @@ class TestDjangoStatic(TestCase):
         """
         
         dir1 = settings.MEDIA_ROOT
-        dir2 = mkdtemp()
-        self.__added_dirs.append(dir2)
+        dir2 = self._mkdir()
         settings.DJANGO_STATIC_MEDIA_ROOTS = [ dir1, dir2 ]
-        dir3 = mkdtemp()
-        self.__added_dirs.append(dir3)
+        dir3 = self._mkdir()
 
         open(dir1 + '/test_A.js', 'w').write("var A=1;")
         open(dir2 + '/test_B.js', 'w').write("var B=1;")
@@ -406,7 +421,20 @@ class TestDjangoStatic(TestCase):
         
         self._test_staticfile_single('/jquery-4.min.js',
                                      'function () { return 1; }',
-                                     media_url=media_url)        
+                                     media_url=media_url)
+                                     
+    def assertFilenamesAlmostEqual(self, name1, name2):
+        # Occasionally we get a failure because the clock ticked to
+        # the next second after the file was rendered.
+        # Thanks https://github.com/slinkp for this contribution!
+        
+        name1, name2 = name1.strip(), name2.strip()
+        timestamp_re = re.compile(r'[^\.]\.([0-9]+)\..*')
+        t1 = int(timestamp_re.search(name1).group(1))
+        t2 = int(timestamp_re.search(name2).group(1))
+        self.assert_(t1 - t2 in (-1, 0, 1),
+                     "Filenames %r and %r timestamps differ by more than 1" % (name1, name2))
+
 
     def _test_staticfile_single(self, filename, code, name_prefix='', save_prefix='',
                                media_url=''):
@@ -424,11 +452,18 @@ class TestDjangoStatic(TestCase):
         bits = filename.rsplit('.', 1)
         now = int(time.time())
         new_filename = bits[0] + '.%s.' % now + bits[-1]
-        self.assertEqual(rendered, media_url + name_prefix + new_filename)
+        self.assertFilenamesAlmostEqual(rendered, media_url + name_prefix + new_filename)
         
         if save_prefix:
             save_dir = os.path.join(os.path.join(settings.MEDIA_ROOT, save_prefix))
-            self.assertTrue(os.path.basename(new_filename) in os.listdir(save_dir))
+            if os.path.basename(new_filename) not in os.listdir(save_dir):
+                # most likely because the clock ticked whilst running the test
+                ts = re.findall('\d{2,10}', new_filename)[0]
+                previous_second = int(ts) - 1
+                other_new_filename = new_filename.replace(ts, str(previous_second))
+                self.assertTrue(os.path.basename(other_new_filename) in os.listdir(save_dir))
+            else:
+                self.assertTrue(os.path.basename(new_filename) in os.listdir(save_dir))
             # The content of the new file should be the same
             new_file = os.listdir(save_dir)[0]
             new_file_path = os.path.join(save_dir, new_file)
@@ -454,7 +489,7 @@ class TestDjangoStatic(TestCase):
         
         # Run it again just to check that it still works
         rendered = template.render(context).strip()
-        self.assertEqual(rendered, media_url + name_prefix + new_filename)
+        self.assertFilenamesAlmostEqual(rendered, media_url + name_prefix + new_filename)
         
         # pretend that time has passed and 10 seconds has lapsed then re-run the
         # rendering again and depending on settings.DEBUG this is noticed
@@ -474,7 +509,7 @@ class TestDjangoStatic(TestCase):
         rendered = template.render(context).strip()
         if settings.DEBUG:
             new_filename = bits[0] + '.%s.' % (now + 10) + bits[1]
-        self.assertEqual(rendered, media_url + name_prefix + new_filename)
+        self.assertFilenamesAlmostEqual(rendered, media_url + name_prefix + new_filename)
         
         if settings.DEBUG:
             
@@ -594,11 +629,27 @@ class TestDjangoStatic(TestCase):
         bits = filename.split('.')
         now = int(time.time())
         new_filename = bits[0] + '.%s.' % now + bits[1]
-        self.assertEqual(rendered, name_prefix + new_filename)
+        self.assertFilenamesAlmostEqual(rendered, name_prefix + new_filename)
         
         if save_prefix:
             save_dir = os.path.join(os.path.join(settings.MEDIA_ROOT, save_prefix))
-            self.assertTrue(os.path.basename(new_filename) in os.listdir(save_dir))
+            if os.path.basename(new_filename) not in os.listdir(save_dir):
+                # Because the clock has ticked in the time this test was running
+                # have to manually check some things. 
+                # This might be what you have on your hands:
+                #  new_filename
+                #  /testingXXX.1291842804.js
+                #  os.path.basename(new_filename)
+                #  testingXXX.1291842804.js
+                #  os.listdir(save_dir)
+                #  ['testingXXX.1291842803.js']
+                ts = re.findall('\d{2,10}', new_filename)[0]
+                previous_second = int(ts) - 1
+                other_new_filename = new_filename.replace(ts, str(previous_second))
+                self.assertTrue(os.path.basename(other_new_filename) in os.listdir(save_dir))
+                
+            else:
+                self.assertTrue(os.path.basename(new_filename) in os.listdir(save_dir))
             # The content of the new file should be smaller
             
             new_file = os.path.join(save_dir, os.listdir(save_dir)[0])
@@ -620,7 +671,7 @@ class TestDjangoStatic(TestCase):
         
         # Run it again just to check that it still works
         rendered = template.render(context).strip()
-        self.assertEqual(rendered, name_prefix + new_filename)
+        self.assertFilenamesAlmostEqual(rendered, name_prefix + new_filename)
         
         # pretend that time has passed and 10 seconds has lapsed then re-run the
         # rendering again and depending on settings.DEBUG this is noticed
@@ -641,7 +692,7 @@ class TestDjangoStatic(TestCase):
         rendered = template.render(context).strip()
         if settings.DEBUG:
             new_filename = bits[0] + '.%s.' % (now + 10) + bits[1]
-        self.assertEqual(rendered, name_prefix + new_filename)
+        self.assertFilenamesAlmostEqual(rendered, name_prefix + new_filename)
         
         if settings.DEBUG:
             
@@ -777,7 +828,7 @@ class TestDjangoStatic(TestCase):
         bits = expect_filename.split('.')
         expect_filename = expect_filename[:-3]
         expect_filename += '.%s%s' % (now, os.path.splitext(filenames[0])[1])
-        self.assertEqual(rendered, name_prefix + expect_filename)
+        self.assertFilenamesAlmostEqual(rendered, name_prefix + expect_filename)
         
         if save_prefix:
             new_filenames_set = os.listdir(os.path.join(settings.MEDIA_ROOT, save_prefix))
@@ -799,7 +850,7 @@ class TestDjangoStatic(TestCase):
         template = Template(template_as_string)
         context = Context()
         rendered = template.render(context).strip()
-        self.assertEqual(rendered, name_prefix + expect_filename)
+        self.assertFilenamesAlmostEqual(rendered, name_prefix + expect_filename)
         
         from posix import stat_result
         def fake_stat(arg):
@@ -822,7 +873,7 @@ class TestDjangoStatic(TestCase):
             # won't notice this until after you restart Django.
             pass
         
-        self.assertEqual(rendered, name_prefix + expect_filename)
+        self.assertFilenamesAlmostEqual(rendered, name_prefix + expect_filename)
         
             
     def test_staticfile_multiple_debug_on(self):
@@ -863,7 +914,7 @@ class TestDjangoStatic(TestCase):
         bits = expect_filename.split('.')
         expect_filename = expect_filename[:-3]
         expect_filename += '.%s%s' % (now, os.path.splitext(filenames[0])[1])
-        self.assertEqual(rendered, name_prefix + expect_filename)
+        self.assertFilenamesAlmostEqual(rendered, name_prefix + expect_filename)
         
         if save_prefix:
             new_filenames_set = os.listdir(os.path.join(settings.MEDIA_ROOT, save_prefix))
@@ -894,7 +945,7 @@ class TestDjangoStatic(TestCase):
         template = Template(template_as_string)
         context = Context()
         rendered = template.render(context).strip()
-        self.assertEqual(rendered, name_prefix + expect_filename)
+        self.assertFilenamesAlmostEqual(rendered, name_prefix + expect_filename)
         
         from posix import stat_result
         def fake_stat(arg):
@@ -917,7 +968,7 @@ class TestDjangoStatic(TestCase):
             # won't notice this until after you restart Django.
             pass
         
-        self.assertEqual(rendered, name_prefix + expect_filename)
+        self.assertFilenamesAlmostEqual(rendered, name_prefix + expect_filename)
         
     def test_staticall_basic(self):
         settings.DEBUG = True
@@ -1071,7 +1122,20 @@ class TestDjangoStatic(TestCase):
 
         expect_filename += '.%s%s' % (now, os.path.splitext(filenames[0])[1])
         
-        self.assertTrue(expect_filename in rendered, expect_filename)
+        if expect_filename not in rendered:
+            # expect_filename is something like
+            # '/testxx_testyy.1291842493.css'
+            # replace that with a more fuzzy matching
+            # This is because the clock might have ticked one second between
+            # when the file was created and when the expect_filename 
+            # variable was prepared.
+            ts = re.findall('\d{2,10}', expect_filename)[0]
+            previous_second = int(ts) - 1
+            other_expect_filename = expect_filename.replace(ts, str(previous_second))
+            self.assertTrue(other_expect_filename in rendered, other_expect_filename)
+            
+        else:
+            self.assertTrue(expect_filename in rendered, expect_filename)
         
         # this should have created a new file
         if save_prefix:
